﻿// system 
using System;
using System.Text;
using System.Diagnostics;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.IO;


// ODA
using Teigha.Runtime;
using Teigha.DatabaseServices;
using Teigha.Geometry;

// Bricsys
using Bricscad.ApplicationServices;
using Bricscad.Runtime;
using Bricscad.EditorInput;
using Bricscad.CivilTin;

// alias
using _AcAp = Bricscad.ApplicationServices;
using _AcDb = Teigha.DatabaseServices;
using _AcGe = Teigha.Geometry;
using _AcEd = Bricscad.EditorInput;

namespace TinDrapePoly
{
	public class TinSurfaceDrape
	{

		// Drape selected polyline to selected TIN surface
		[CommandMethod("TinDrapePoly")]
		static public void TinDrapePoly()
		{
			TINDrapePoly();
		}


		public static void TINDrapePoly()
		{
			Editor editor = _AcAp.Application.DocumentManager.MdiActiveDocument.Editor;
			try
			{
				_AcEd.PromptEntityOptions peo = new _AcEd.PromptEntityOptions("Select TIN Surface: ");
				peo.SetRejectMessage("Not a TIN surface!");
				peo.AddAllowedClass(typeof(CivilTinSurface), true);
				_AcEd.PromptEntityResult per = editor.GetEntity(peo);

				_AcEd.PromptEntityOptions peo1 = new _AcEd.PromptEntityOptions("Select Polyline: ");
				peo.SetRejectMessage("Not a Polyline!");
				peo.AddAllowedClass(typeof(Polyline), true);
				_AcEd.PromptEntityResult per1 = editor.GetEntity(peo1);

				if (per.Status == _AcEd.PromptStatus.OK && per1.Status == _AcEd.PromptStatus.OK)
				{
					_AcDb.Database database = _AcDb.HostApplicationServices.WorkingDatabase;
					_AcDb.TransactionManager manager = database.TransactionManager;

					using (_AcDb.Transaction action = manager.StartTransaction())
					{
						CivilTinSurface tinSurface = action.GetObject(per.ObjectId, _AcDb.OpenMode.ForRead) as CivilTinSurface;
						Polyline poly = action.GetObject(per1.ObjectId, _AcDb.OpenMode.ForRead) as Polyline;
						Point3dCollection points = new Point3dCollection();

						if (tinSurface != null)
						{
							// Use a for loop to get each vertex, one by one
							int vn = poly.NumberOfVertices;
							for (int i = 0; i < vn; i++)
							{
								// Could also get the 3D point here
								Point3d pt = poly.GetPoint3dAt(i);
								points.Add(new Point3d(pt.X, pt.Y, pt.Z));
							}

							/*
							The drape function projects a 3d polyline onto the given TIN Surface.
							The result can be:
							 - empty (given points (polyline vertices) and the surface do not intersect,
							   if both would be projected onto XY plane)
							 - one Point3dCollection (all the polyline vertices can be projected without
							   any discontinuites)
							 - multiple resulting Point3dCollections (some parts of the polyline
							   can be projected on the surface, while others "fall" of the surface
							   or are inside the hide boundaries)
							*/

							List<Point3dCollection> drapedPointGroups = tinSurface.Drape(points);
							foreach (Point3dCollection drapedPoints in drapedPointGroups)
							{
								Polyline3d polyline = new Polyline3d(Poly3dType.SimplePoly, drapedPoints, false);
								polyline.ColorIndex = 1;
								AddToModelSpace(polyline);
							}

						}
						action.Commit();
					}
				}
			}
			catch (System.Exception ex)
			{
				_AcAp.Application.ShowAlertDialog(
					string.Format("\nError: {0}\nStackTrace: {1}", ex.Message, ex.StackTrace));
			}

		}

		//helper Function to add items to ModelSpace;
		public static ObjectIdCollection AddToModelSpace(params Entity[] list)
		{
			ObjectIdCollection ids = new ObjectIdCollection();
			Database database = HostApplicationServices.WorkingDatabase;
			_AcDb.TransactionManager manager = database.TransactionManager;
			using (Transaction action = manager.StartTransaction())
			{
				BlockTable blockTable = action.GetObject(database.BlockTableId, OpenMode.ForRead) as BlockTable;
				if (blockTable == null)
					throw new System.NullReferenceException("blockTable == null");

				BlockTableRecord blockTableRecord = action.GetObject(blockTable[BlockTableRecord.ModelSpace], OpenMode.ForWrite) as BlockTableRecord;
				if (blockTableRecord == null)
					throw new System.NullReferenceException("blockTableRecord == null");

				foreach (Entity ent in list)
				{
					if (ent != null)
					{
						ids.Add(blockTableRecord.AppendEntity(ent));
						action.AddNewlyCreatedDBObject(ent, true);
					}
				}
				action.Commit();
			}
			return ids;
		}
	}

}
